function td_local(filename, azimuth, elevation, data_set, gain_factor)
% td_local(filename, azimuth, elevation, data_set, gain_factor)
%Function to use standard convolution and MIT/KEMAR filters to
% localize a wave file at the given azimuth elevation.   
% Prepends a string 'ref_' to the given filename and writes the
% results to disk.
% Note: data_set is 'L', 'R', or 'H' -- see KEMAR FAQ for details.
% Gain_factor should be (0, 1]
% pfh 4/26/99
% phubbard@computer.org and umland@math.unm.edu 

% Save starting time
begin_time = clock;
begin_flops = flops;

% Construct output filename
out_file = sprintf('ref_%s', filename);

% Sample rate of filter
sample_rate = 44100;

% Call the Gardner function to load the time-domain HRTF
% Produces a 2x512 array, left channel in 1, right in 2
disp 'Reading HRTF from file'
hrtf = readhrtf(elevation, azimuth, data_set);

disp 'Reading sound file'
sound = wavread(filename);

disp 'Convolving'
snd_out1 = conv(sound, hrtf(1,:));
snd_out2 = conv(sound, hrtf(2,:));

disp 'Normalizing and saving results'
snd_out = zeros(2, length(snd_out1));
snd_out(1,:) = snd_out1';
snd_out(2,:) = snd_out2';

% Find max value for normalizing
smax = abs(max(max(snd_out)));

% Normalize to same (should yield +-1)
snd_out = (snd_out' * (1 / smax)) * gain_factor;

% Save resulting sound
wavwrite(snd_out, sample_rate, out_file);

% Show how hard we worked
end_time = clock;

sprintf('Elapsed time: %s seconds Flops: %d', etime(end_time, begin_time), ...
	flops - begin_flops)

  
