function wd_local(in_file, wqmf, final_gain, res_level)
% function wd_local(in_file, wqmf, final_gain, res_level)
% File to actually apply the operator to a wave file.
% Supposed to sound localized!
% pfh/ku 4-99
% phubbard@computer.org and umland@math.unm.edu 

% Save starting time
begin_time = clock;
begin_flops = flops;

% Build output filename by prepending a wd_
out_file = sprintf('wd_%s', in_file);

% Printf debugging
sprintf('Function: WD localization, filename %s, L=%d', in_file, res_level)

% --------------
% Load NS-form of operators from file
disp 'Loading matrices'
load ns_r.mat
load ns_l.mat
disp 'Done loading'
disp 'Loading data file and convolving'

% Global sampling rate
sample_rate = 44100;

% Load data file
tmp = wavread(in_file);
s_len = length(tmp);

% Add 512 zeros to start of data file - wraparound effects
% Also add a bunch of zeros onto the end
s_len = s_len + 512;
sound = zeros(1, s_len + 4096);

% Copy real data in
sound(513:s_len) = tmp;
clear tmp

tbuff=zeros(1,2048);
obuff=zeros(1,2048);

% Wave output vector (stereo)
snd_out = zeros(s_len,2);

% Figure out how many times we have to operate
iterations = floor((s_len+512) / 1024);
remainder = mod((s_len+512), 1024);

% If have a remainder, then need one more iteration
if (remainder > 0)
  iterations = iterations + 1;
end

for x=1:iterations
  % Compute left index
  idx = (x - 1) * 1024 + 1;

  % Copy data into working buffer
  tbuff = sound(idx: idx+2047);
    
  % Apply localization operator - also takes wavelet xforms
  % This could be done with a call to NonStdWaveMult, but that 
  % has a bug in it, so the next three lines are the corrected
  % version.
  % We also save time but not recomputing FNWT(tbuff) for the other 
  % channel.
  
  % First of all, take NSWT of input sound block
  nx = FNWT_PO(tbuff, res_level, wqmf);
  
  % Build Miner operator - the location of the non-unity entries
  % determines which scale we are modifying. Since this is in NSF,
  % see the BCR paper for details as to the new locations in WD
  % vector. 
  % Since her operator is a diagonal, start with a sparse diagonal
  % identity matrix, and then modify the non-unity coeffs.
  n_oper = speye(4096);
  for i=347:512
    n_oper(i,i) = 16.0;
  end
  
  % Convolve/apply localization
  ny = ns_l * nx;
  
  % Apply Miner operator
  ny = n_oper * ny;

  % Inverse NS-wavelet xform
  obuff = INWT_PO(ny', res_level, wqmf);
  
  % Save result from middle half of buffer
  snd_out(idx: idx + 1023,1) = obuff(513:1536);
  
  % other ear - already have WD-data, just do mult & inverse
  % Convolve
  ny = ns_r * nx;
  
  % Apply Miner operator
  ny = n_oper * ny;
  
  % Inverse
  obuff = INWT_PO(ny', res_level, wqmf);
  
  % Save 
  snd_out(idx: idx + 1023,2) = obuff(513:1536);
end

% Find max value for normalizing
smax = abs(max(max(snd_out)));

% If max value is zero, then we have a problem somewhere!
if(smax == 0)
  disp 'Probably an invalid threshold - no output data greater than zero!'
  smax = 1;
end
  
% Normalize to same (should yield +-1)
snd_out = (snd_out' * (1 / smax)) * final_gain;

% Save resulting sound
wavwrite(snd_out', sample_rate, out_file);

% Show how hard we worked
end_time = clock;

sprintf('Elapsed time: %s seconds MFlops: %d', etime(end_time, begin_time), ...
	(flops - begin_flops) / 1e6)

    
